@@ -4,14 +4,15 @@ usage: subaligner [-h] [-m {single,dual,script,shift,transcribe}] [-v VIDEO_PATH
                   [-sil {afr,amh,ara,arg,asm,aze,ben,bos,bul,cat,ces,cmn,cym,dan,deu,ell,eng,epo,est,eus,fas,fin,fra,gla,gle,glg,grc,grn,guj,heb,hin,hrv,hun,hye,ina,ind,isl,ita,jbo,jpn,kal,kan,kat,kir,kor,kur,lat,lav,lfn,lit,mal,mar,mkd,mlt,msa,mya,nah,nep,nld,nor,ori,orm,pan,pap,pol,por,ron,rus,sin,slk,slv,spa,sqi,srp,swa,swe,tam,tat,tel,tha,tsn,tur,ukr,urd,vie,yue,zho}]
                   [-fos] [-tod TRAINING_OUTPUT_DIRECTORY] [-o OUTPUT] [-t TRANSLATE] [-os OFFSET_SECONDS]
                   [-ml {afr,amh,ara,arg,asm,aze,ben,bos,bul,cat,ces,cmn,cym,dan,deu,ell,eng,epo,est,eus,fas,fin,fra,gla,gle,glg,grc,grn,guj,heb,hin,hrv,hun,hye,ina,ind,isl,ita,jbo,jpn,kal,kan,kat,kir,kor,kur,lat,lav,lfn,lit,mal,mar,mkd,mlt,msa,mya,nah,nep,nld,nor,ori,orm,pan,pap,pol,por,ron,rus,sin,slk,slv,spa,sqi,srp,swa,swe,tam,tat,tel,tha,tsn,tur,ukr,urd,vie,yue,zho}]
-                  [-mr {whisper}] [-mf {tiny,tiny.en,small,medium,medium.en,base,base.en,large-v1,large-v2,large}] [-lgs] [-d] [-q] [-ver]
+                  [-mr {whisper}] [-mf {tiny,tiny.en,small,medium,medium.en,base,base.en,large-v1,large-v2,large-v3,large,turbo}] [-ip INITIAL_PROMPT] [-mcl MAX_CHAR_LENGTH]
+                  [-tr {helsinki-nlp,whisper,facebook-mbart,facebook-m2m100}] [-tf TRANSLATION_FLAVOUR] [-mpt MEDIA_PROCESS_TIMEOUT] [-sat SEGMENT_ALIGNMENT_TIMEOUT] [-upp] [-wtc] [-lgs] [-d] [-q] [-ver]
 
 Subaligner command line interface
 
 optional arguments:
   -h, --help            show this help message and exit
   -s SUBTITLE_PATH [SUBTITLE_PATH ...], --subtitle_path SUBTITLE_PATH [SUBTITLE_PATH ...]
-                        File path or URL to the subtitle file (Extensions of supported subtitles: .ssa, .vtt, .srt, .txt, .smi, .ytt, .sub, .xml, .sbv, .ass, .sami, .scc, .tmp, .stl, .ttml, .dfxp) or selector for the embedded subtitle (e.g., embedded:page_num=888 or embedded:stream_index=0)
+                        File path or URL to the subtitle file (Extensions of supported subtitles: .ssa, .tmp, .srt, .sbv, .stl, .json, .sami, .ttml, .smi, .txt, .scc, .sub, .ass, .vtt, .xml, .ytt, .dfxp) or selector for the embedded subtitle (e.g., embedded:page_num=888 or embedded:stream_index=0)
   -l MAX_LOGLOSS, --max_logloss MAX_LOGLOSS
                         Max global log loss for alignment
   -so, --stretch_on     Switch on stretch on subtitles)
@@ -29,10 +30,26 @@ optional arguments:
                         Offset by which the subtitle will be shifted
   -ml {afr,amh,ara,arg,asm,aze,ben,bos,bul,cat,ces,cmn,cym,dan,deu,ell,eng,epo,est,eus,fas,fin,fra,gla,gle,glg,grc,grn,guj,heb,hin,hrv,hun,hye,ina,ind,isl,ita,jbo,jpn,kal,kan,kat,kir,kor,kur,lat,lav,lfn,lit,mal,mar,mkd,mlt,msa,mya,nah,nep,nld,nor,ori,orm,pan,pap,pol,por,ron,rus,sin,slk,slv,spa,sqi,srp,swa,swe,tam,tat,tel,tha,tsn,tur,ukr,urd,vie,yue,zho}, --main_language {afr,amh,ara,arg,asm,aze,ben,bos,bul,cat,ces,cmn,cym,dan,deu,ell,eng,epo,est,eus,fas,fin,fra,gla,gle,glg,grc,grn,guj,heb,hin,hrv,hun,hye,ina,ind,isl,ita,jbo,jpn,kal,kan,kat,kir,kor,kur,lat,lav,lfn,lit,mal,mar,mkd,mlt,msa,mya,nah,nep,nld,nor,ori,orm,pan,pap,pol,por,ron,rus,sin,slk,slv,spa,sqi,srp,swa,swe,tam,tat,tel,tha,tsn,tur,ukr,urd,vie,yue,zho}
                         Target video's main language as an ISO 639-3 language code [https://en.wikipedia.org/wiki/List_of_ISO_639-3_codes]
-  -mr {whisper}, --llm_recipe {whisper}
+  -mr {whisper}, --transcription_recipe {whisper}
                         LLM recipe used for transcribing video files
-  -mf {tiny,tiny.en,small,medium,medium.en,base,base.en,large-v1,large-v2,large}, --llm_flavour {tiny,tiny.en,small,medium,medium.en,base,base.en,large-v1,large-v2,large}
-                        Flavour variation for a specific LLM recipe
+  -mf {tiny,tiny.en,small,medium,medium.en,base,base.en,large-v1,large-v2,large-v3,large,turbo}, --transcription_flavour {tiny,tiny.en,small,medium,medium.en,base,base.en,large-v1,large-v2,large-v3,large,turbo}
+                        Flavour variation for a specific LLM recipe supporting transcription
+  -ip INITIAL_PROMPT, --initial_prompt INITIAL_PROMPT
+                        Optional text to provide the transcribing context or specific phrases
+  -mcl MAX_CHAR_LENGTH, --max_char_length MAX_CHAR_LENGTH
+                        Maximum number of characters for each generated subtitle segment
+  -tr {helsinki-nlp,whisper,facebook-mbart,facebook-m2m100}, --translation_recipe {helsinki-nlp,whisper,facebook-mbart,facebook-m2m100}
+                        LLM recipe used for translating subtitles
+  -tf TRANSLATION_FLAVOUR, --translation_flavour TRANSLATION_FLAVOUR
+                        Flavour variation for a specific LLM recipe supporting translation
+  -mpt MEDIA_PROCESS_TIMEOUT, --media_process_timeout MEDIA_PROCESS_TIMEOUT
+                        Maximum waiting time in seconds when processing media files
+  -sat SEGMENT_ALIGNMENT_TIMEOUT, --segment_alignment_timeout SEGMENT_ALIGNMENT_TIMEOUT
+                        Maximum waiting time in seconds when aligning each segment
+  -upp, --use_prior_prompting
+                        Whether to use the previous subtitle cue as the current prompt.
+  -wtc, --word_time_codes
+                        Whether to output time codes for each word in the subtitle file.
   -lgs, --languages     Print out language codes used for stretch and translation
   -d, --debug           Print out debugging information
   -q, --quiet           Switch off logging information
@@ -50,7 +67,6 @@ import sys
 import traceback
 import os
 import tempfile
-import pkg_resources
 
 
 def main():
@@ -152,22 +168,73 @@ def main():
         choices=Utils.get_stretch_language_codes(),
         help="Target video's main language as an ISO 639-3 language code [https://en.wikipedia.org/wiki/List_of_ISO_639-3_codes]",
     )
+    from subaligner.llm import TranscriptionRecipe
+    from subaligner.llm import WhisperFlavour
     parser.add_argument(
         "-mr",
-        "--llm_recipe",
+        "--transcription_recipe",
         type=str.lower,
-        default="whisper",
-        choices=["whisper"],
+        default=TranscriptionRecipe.WHISPER.value,
+        choices=[r.value for r in TranscriptionRecipe],
         help="LLM recipe used for transcribing video files"
     )
     parser.add_argument(
         "-mf",
-        "--llm_flavour",
+        "--transcription_flavour",
         type=str.lower,
-        default="small",
-        choices=["tiny", "tiny.en", "small", "medium", "medium.en", "base", "base.en", "large-v1", "large-v2", "large"],
-        help="Flavour variation for a specific LLM recipe"
+        default=WhisperFlavour.SMALL.value,
+        choices=[wf.value for wf in WhisperFlavour],
+        help="Flavour variation for a specific LLM recipe supporting transcription"
     )
+    parser.add_argument(
+        "-ip",
+        "--initial_prompt",
+        type=str,
+        default=None,
+        help="Optional text to provide the transcribing context or specific phrases"
+    )
+    parser.add_argument(
+        "-mcl",
+        "--max_char_length",
+        type=int,
+        default=None,
+        help="Maximum number of characters for each generated subtitle segment"
+    )
+    from subaligner.llm import TranslationRecipe
+    from subaligner.llm import HelsinkiNLPFlavour
+    parser.add_argument(
+        "-tr",
+        "--translation_recipe",
+        type=str.lower,
+        default=TranslationRecipe.HELSINKI_NLP.value,
+        choices=[r.value for r in TranslationRecipe],
+        help="LLM recipe used for translating subtitles"
+    )
+    parser.add_argument(
+        "-tf",
+        "--translation_flavour",
+        type=str.lower,
+        default=None,
+        help="Flavour variation for a specific LLM recipe supporting translation"
+    )
+    parser.add_argument(
+        "-mpt",
+        "--media_process_timeout",
+        type=int,
+        default=180,
+        help="Maximum waiting time in seconds when processing media files"
+    )
+    parser.add_argument(
+        "-sat",
+        "--segment_alignment_timeout",
+        type=int,
+        default=60,
+        help="Maximum waiting time in seconds when aligning each segment"
+    )
+    parser.add_argument("-upp", "--use_prior_prompting", action="store_true",
+                        help="Whether to use the previous subtitle cue as the current prompt.")
+    parser.add_argument("-wtc", "--word_time_codes", action="store_true",
+                        help="Whether to output time codes for each word in the subtitle file.")
     parser.add_argument("-lgs", "--languages", action="store_true",
                         help="Print out language codes used for stretch and translation")
     parser.add_argument("-d", "--debug", action="store_true",
@@ -192,7 +259,8 @@ def main():
         parser.print_usage()
         sys.exit(21)
     elif FLAGS.mode == "transcribe":
-        FLAGS.subtitle_path = ["{}.srt".format(tempfile.mkstemp()[1])]
+        if not FLAGS.subtitle_path:
+            FLAGS.subtitle_path = [tempfile.mkstemp(suffix="_transcribe_temp.srt")[1]]
     if FLAGS.mode in ["single", "dual", "script", "transcribe"]:
         for subtitle_path in FLAGS.subtitle_path:
             if FLAGS.video_path == "":
@@ -221,11 +289,16 @@ def main():
                     parser.print_usage()
                     sys.exit(21)
             if FLAGS.translate is not None or FLAGS.mode == "transcribe":
-                if "transformers" not in {pkg.key for pkg in pkg_resources.working_set}:
+                try:
+                    import transformers
+                except ModuleNotFoundError:
                     print('ERROR: Alignment has been configured to use language models. Please install "subaligner[llm]" and run your command again.')
                     sys.exit(21)
             if FLAGS.stretch_on or FLAGS.mode == "script":
-                if "aeneas" not in {pkg.key for pkg in pkg_resources.working_set}:
+                try:
+                    import aeneas
+                    import dtw
+                except ModuleNotFoundError:
                     print('ERROR: Alignment has been configured to use extra features. Please install "subaligner[stretch]" and run your command again.')
                     sys.exit(21)
 
@@ -274,7 +347,7 @@ def main():
                         sys.exit(21)
 
                 voice_probabilities = None
-                predictor = Predictor()
+                predictor = Predictor(media_process_timeout=FLAGS.media_process_timeout, segment_alignment_timeout=FLAGS.segment_alignment_timeout)
                 if FLAGS.mode == "single":
                     aligned_subs, audio_file_path, voice_probabilities, frame_rate = predictor.predict_single_pass(
                         video_file_path=local_video_path,
@@ -295,11 +368,24 @@ def main():
                         video_file_path=local_video_path,
                         subtitle_file_path=local_subtitle_path,
                         stretch_in_lang=stretch_in_lang,
+                        with_word_time_codes=FLAGS.word_time_codes,
                     )
                 elif FLAGS.mode == "transcribe":
                     from subaligner.transcriber import Transcriber
-                    transcriber = Transcriber(recipe=FLAGS.llm_recipe, flavour=FLAGS.llm_flavour)
-                    subtitle, frame_rate = transcriber.transcribe(local_video_path, stretch_in_lang)
+                    transcriber = Transcriber(recipe=FLAGS.transcription_recipe, flavour=FLAGS.transcription_flavour)
+                    if "_transcribe_temp" in local_subtitle_path:
+                        subtitle, frame_rate = transcriber.transcribe(video_file_path=local_video_path,
+                                                                      language_code=stretch_in_lang,
+                                                                      initial_prompt=FLAGS.initial_prompt,
+                                                                      max_char_length=FLAGS.max_char_length,
+                                                                      with_word_time_codes=FLAGS.word_time_codes)
+                    else:
+                        subtitle, frame_rate = transcriber.transcribe_with_subtitle_as_prompts(video_file_path=local_video_path,
+                                                                                               subtitle_file_path=local_subtitle_path,
+                                                                                               language_code=stretch_in_lang,
+                                                                                               max_char_length=FLAGS.max_char_length,
+                                                                                               use_prior_prompting=FLAGS.use_prior_prompting,
+                                                                                               with_word_time_codes=FLAGS.word_time_codes,)
                     aligned_subs = subtitle.subs
                 else:
                     print("ERROR: Unknown mode {}".format(FLAGS.mode))
@@ -312,8 +398,8 @@ def main():
                 if FLAGS.translate is not None:
                     from subaligner.translator import Translator
                     source, target = FLAGS.translate.split(",")
-                    translator = Translator(source, target)
-                    aligned_subs = translator.translate(aligned_subs)
+                    translator = Translator(src_language=source, tgt_language=target, recipe=FLAGS.translation_recipe, flavour=FLAGS.translation_flavour)
+                    aligned_subs = translator.translate(aligned_subs, local_video_path, (source, target))
                     Subtitle.save_subs_as_target_format(aligned_subs, local_subtitle_path, aligned_subtitle_path,
                                                         frame_rate, "utf-8")
                 elif FLAGS.mode == "transcribe":
@@ -376,7 +462,7 @@ def _remove_tmp_files(video_path, subtitle_path, local_video_path, local_subtitl
         os.remove(local_video_path)
     if subtitle_path.lower().startswith("http") and os.path.exists(local_subtitle_path):
         os.remove(local_subtitle_path)
-    if mode == "transcribe" and os.path.exists(local_subtitle_path):
+    if mode == "transcribe" and os.path.exists(local_subtitle_path) and "_transcribe_temp" in local_subtitle_path:
         os.remove(local_subtitle_path)
 
 
