@@ -10,5 +10,9 @@ class NoFrameRateException(Exception):
     """ An exception raised due to frame rate not found."""
 
 
+class TranslationException(Exception):
+    """ An exception raised due to translation failures."""
+
+
 class TranscriptionException(Exception):
     """ An exception raised due to transcription failures."""
