@@ -1,6 +1,5 @@
 import subprocess
 import os
-import threading
 import traceback
 import tempfile
 import shutil
@@ -16,6 +15,7 @@ from .embedder import FeatureEmbedder
 from .exception import TerminalException
 from .exception import NoFrameRateException
 from .logger import Logger
+from .utils import Utils
 
 TEMP_DIR_PATH = tempfile.mkdtemp()
 
@@ -28,6 +28,9 @@ def clear_temp(*_):
 class MediaHelper(object):
     """ Utility for processing media assets including audio, video and
     subtitle files.
+
+    Arguments:
+        media_process_timeout {int} -- The timeout in seconds on processing media files.
     """
 
     FFMPEG_BIN = os.getenv("FFMPEG_PATH") or os.getenv("ffmpeg_path") or "ffmpeg"
@@ -38,24 +41,30 @@ class MediaHelper(object):
     __MIN_GAP_IN_SECS = (
         1  # minimum gap in seconds between consecutive subtitle during segmentation
     )
-    __CMD_TIME_OUT = 180  # time out for subprocess
 
     atexit.register(clear_temp)
     signal.signal(signal.SIGTERM, clear_temp)
 
-    def __init__(self):
+    def __init__(self, media_process_timeout: int = 180) -> None:
         self.__LOGGER = Logger().get_logger(__name__)
+        self.__media_process_timeout = media_process_timeout
 
     def extract_audio(self, video_file_path, decompress: bool = False, freq: int = 16000) -> str:
         """Extract audio track from the video file and save it to a WAV file.
 
         Arguments:
             video_file_path {string} -- The input video file path.
+
         Keyword Arguments:
             decompress {bool} -- Extract WAV if True otherwise extract AAC (default: {False}).
             freq {int} -- The audio sample frequency (default: {16000}).
+
         Returns:
-            string -- The file path of the extracted audio.
+            str: The file path of the extracted audio.
+
+        Raises:
+            TerminalException: If audio extraction is interrupted by user hitting the interrupt key or timed out.
+            Exception: Thrown when any other exceptions occur.
         """
 
         basename = os.path.basename(video_file_path)
@@ -73,12 +82,12 @@ class MediaHelper(object):
             )
 
         command = (
-            "{0} -y -xerror -i '{1}' -ac 2 -ar {2} -vn '{3}'".format(
-                self.FFMPEG_BIN, video_file_path, freq, audio_file_path
+            "{0} -y -xerror -i {1} -ac 2 -ar {2} -vn {3}".format(
+                self.FFMPEG_BIN, Utils.double_quoted(video_file_path), freq, Utils.double_quoted(audio_file_path)
             )
             if decompress
-            else "{0} -y -xerror -i '{1}' -vn -acodec copy '{2}'".format(
-                self.FFMPEG_BIN, video_file_path, audio_file_path
+            else "{0} -y -xerror -i {1} -vn -acodec copy {2}".format(
+                self.FFMPEG_BIN, Utils.double_quoted(video_file_path), Utils.double_quoted(audio_file_path)
             )
         )
         with subprocess.Popen(
@@ -92,7 +101,7 @@ class MediaHelper(object):
         ) as process:
             try:
                 self.__LOGGER.debug("[{}] Running: {}".format(process.pid, command))
-                _, std_err = process.communicate(timeout=self.__CMD_TIME_OUT)
+                _, std_err = process.communicate(timeout=self.__media_process_timeout)
                 self.__LOGGER.debug("[{}] {}".format(process.pid, std_err))
                 if process.returncode != 0:
                     self.__LOGGER.error("[{}] Cannot extract audio from video: {}\n{}"
@@ -143,7 +152,7 @@ class MediaHelper(object):
             end {string} -- The end time (e.g., 00:00:10,230).
 
         Returns:
-            float -- The duration in seconds.
+            Optional[float]: The duration in seconds.
         """
 
         if start is None:
@@ -170,7 +179,11 @@ class MediaHelper(object):
             end {string} -- The end time (e.g., 00:00:10,230) (default: {None}).
 
         Returns:
-            tuple -- The file path to the extracted audio and its duration.
+            tuple: The file path to the extracted audio and its duration.
+
+        Raises:
+            TerminalException: If audio extraction is interrupted by user hitting the interrupt key or timed out.
+            Exception: Thrown when any other exceptions occur.
         """
         segment_duration = self.get_duration_in_seconds(start, end)
         basename = os.path.basename(audio_file_path)
@@ -182,12 +195,12 @@ class MediaHelper(object):
 
         if end is not None:
             duration = self.get_duration_in_seconds(start, end)
-            command = "{0} -y -xerror -i '{1}' -ss {2} -t {3} -acodec copy '{4}'".format(
-                self.FFMPEG_BIN, audio_file_path, start, duration, segment_path
+            command = "{0} -y -xerror -i {1} -ss {2} -t {3} -acodec copy {4}".format(
+                self.FFMPEG_BIN, Utils.double_quoted(audio_file_path), start, duration, Utils.double_quoted(segment_path)
             )
         else:
-            command = "{0} -y -xerror -i '{1}' -ss {2} -acodec copy '{3}'".format(
-                self.FFMPEG_BIN, audio_file_path, start, segment_path
+            command = "{0} -y -xerror -i {1} -ss {2} -acodec copy {3}".format(
+                self.FFMPEG_BIN, Utils.double_quoted(audio_file_path), start, Utils.double_quoted(segment_path)
             )
         with subprocess.Popen(
             shlex.split(command),
@@ -199,7 +212,7 @@ class MediaHelper(object):
         ) as process:
             self.__LOGGER.debug("[{}] Running: {}".format(process.pid, command))
             try:
-                _, std_err = process.communicate(timeout=self.__CMD_TIME_OUT)
+                _, std_err = process.communicate(timeout=self.__media_process_timeout)
                 self.__LOGGER.debug("[{}] {}".format(process.pid, std_err))
                 if process.returncode != 0:
                     self.__LOGGER.error("[{}] Cannot clip audio: {} Return Code: {}\n{}"
@@ -220,7 +233,7 @@ class MediaHelper(object):
                 if os.path.exists(segment_path):
                     os.remove(segment_path)
                 raise TerminalException(
-                    "Timeout on extracting audio from audio: {} after {} seconds".format(audio_file_path, self.__CMD_TIME_OUT)
+                    "Timeout on extracting audio from audio: {} after {} seconds".format(audio_file_path, self.__media_process_timeout)
                 ) from e
             except Exception as e:
                 self.__LOGGER.error(
@@ -258,7 +271,7 @@ class MediaHelper(object):
             subs {list} -- A list of SupRip cues.
 
         Returns:
-            tuple -- A list of start times, a list of end times and a list of grouped SubRip files.
+            tuple: A list of start times, a list of end times and a list of grouped SubRip files.
         """
 
         local_subs = self.__preprocess_subs(subs)
@@ -309,14 +322,20 @@ class MediaHelper(object):
 
         Arguments:
             file_path {string} -- The input audiovisual file path.
+
         Returns:
-            float -- The frame rate
+            float: The frame rate
+
+        Raises:
+            TerminalException: If frame rate extraction is interrupted by user hitting the interrupt key or timed out.
+            NoFrameRateException: If no frame rate is detected on the input audiovisual file.
+            Exception: Thrown when any other exceptions occur.
         """
 
         discarded = "NUL:" if os.name == "nt" else "/dev/null"
 
         with subprocess.Popen(
-                shlex.split("{0} -i '{1}' -t 00:00:10 -f null {2}".format(self.FFMPEG_BIN, file_path, discarded)),
+                shlex.split("{0} -i {1} -t 00:00:10 -f null {2}".format(self.FFMPEG_BIN, Utils.double_quoted(file_path), discarded)),
                 shell=False,
                 stderr=subprocess.PIPE,
                 close_fds=True,
@@ -334,7 +353,7 @@ class MediaHelper(object):
                     bufsize=1,
             ) as process:
                 try:
-                    std_out, std_err = process.communicate(timeout=self.__CMD_TIME_OUT)
+                    std_out, std_err = process.communicate(timeout=self.__media_process_timeout)
                     if process.returncode != 0:
                         self.__LOGGER.warning("[{}] Cannot extract the frame rate from video: {}\n{}".format(process.pid, file_path, std_err))
                         raise NoFrameRateException(
@@ -376,8 +395,9 @@ class MediaHelper(object):
         Arguments:
             subs {list} -- A list of SupRip cues.
             minimum_segment_duration {float} -- The minimum duration in seconds for each output subtitle cue.
+
         Returns:
-            list -- A list of new SupRip cues after fragmentation.
+            list: A list of new SupRip cues after fragmentation.
         """
         new_segment = []
         new_segment_index = 0
