@@ -4,29 +4,52 @@ import importlib
 import psutil
 import numpy as np
 import tensorflow as tf
-import tensorflow.keras.optimizers as tf_optimizers
 
-from typing import Tuple, Optional, Any, List, Generator
-from tensorflow.keras.layers import (
-    Dense,
-    Input,
-    LSTM,
-    Conv1D,
-    MaxPooling1D,
-    Dropout,
-    Activation,
-    BatchNormalization,
-    Bidirectional,
-)
+from typing import Tuple, Optional, List, Generator
+if tf.__version__ >= "2.16.0":
+    import tf_keras.optimizers as tf_optimizers
+    from tf_keras.layers import (
+        Dense,
+        Input,
+        LSTM,
+        Conv1D,
+        MaxPooling1D,
+        Dropout,
+        Activation,
+        BatchNormalization,
+        Bidirectional,
+    )
 
-from tensorflow.keras.callbacks import (
-    EarlyStopping,
-    ModelCheckpoint,
-    TensorBoard,
-    CSVLogger,
-)
-from tensorflow.keras.models import Model, load_model, save_model
-from tensorflow.keras import backend as K
+    from tf_keras.callbacks import (
+        EarlyStopping,
+        ModelCheckpoint,
+        TensorBoard,
+        CSVLogger,
+    )
+    from tf_keras.models import Model, load_model, save_model
+    from tf_keras import backend as K
+else:
+    import keras.optimizers as tf_optimizers
+    from keras.layers import (
+        Dense,
+        Input,
+        LSTM,
+        Conv1D,
+        MaxPooling1D,
+        Dropout,
+        Activation,
+        BatchNormalization,
+        Bidirectional,
+    )
+
+    from keras.callbacks import (
+        EarlyStopping,
+        ModelCheckpoint,
+        TensorBoard,
+        CSVLogger,
+    )
+    from keras.models import Model, load_model, save_model
+    from keras import backend as K
 from .utils import Utils
 from .logger import Logger
 from .exception import TerminalException
@@ -38,6 +61,17 @@ class Network(object):
     """ Network factory creates DNNs.
     Not thread safe since the session of keras_backend is global.
     Only factory methods are allowed when generating DNN objects.
+
+    Arguments:
+        secret {object} -- A hash only known by factory methods.
+        input_shape {tuple} -- A shape tuple (integers), not including the batch size.
+        hyperparameters {Hyperparameters} -- A configuration for hyperparameters used for training.
+        model_path {string} -- The path to the model file.
+        backend {string} -- The tensor manipulation backend (default: {tensorflow}). Only tensorflow is supported
+                            by TF 2 and this parameter is here only for a historical reason.
+
+    Raises:
+        ValueError: Thrown when the network type is not supported.
     """
 
     LSTM = "lstm"
@@ -45,7 +79,6 @@ class Network(object):
     CONV_1D = "conv_1d"
     TYPES = [LSTM, BI_LSTM, CONV_1D]
 
-    __LOGGER = Logger().get_logger(__name__)
     __secret = object()
     __UNKNOWN = "unknown"
 
@@ -57,19 +90,6 @@ class Network(object):
         model_path: Optional[str] = None,
         backend: str = "tensorflow"
     ) -> None:
-        """ Network object initialiser used by factory methods.
-
-        Arguments:
-            secret {object} -- A hash only known by factory methods.
-            input_shape {tuple} -- A shape tuple (integers), not including the batch size.
-            hyperparameters {Hyperparameters} -- A configuration for hyperparameters used for training.
-            model_path {string} -- The path to the model file.
-            backend {string} -- The tensor manipulation backend (default: {tensorflow}). Only tensorflow is supported
-                                by TF 2 and this parameter is here only for a historical reason.
-        Raises:
-            NotImplementedError -- Thrown when any network attributes are modified.
-        """
-
         assert (
             secret == Network.__secret
         ), "Only factory methods are supported when creating instances"
@@ -99,6 +119,7 @@ class Network(object):
 
         self.__n_type = hyperparameters.network_type
         self.hyperparameters = hyperparameters
+        self.__LOGGER = Logger().get_logger(__name__)
 
     @classmethod
     def get_network(cls, input_shape: Tuple, hyperparameters: Hyperparameters) -> "Network":
@@ -109,7 +130,7 @@ class Network(object):
             hyperparameters {Hyperparameters} -- A configuration for hyperparameters used for training.
 
         Returns:
-            Network -- A constructed network object.
+            Network: A constructed network object.
         """
 
         return cls(
@@ -125,6 +146,9 @@ class Network(object):
         Arguments:
             model_path {string} -- The path to the model file.
             hyperparameters {Hyperparameters} -- A configuration for hyperparameters used for training.
+
+        Returns:
+            Network: The model network.
         """
 
         hp = hyperparameters.clone()
@@ -161,18 +185,18 @@ class Network(object):
             hyperparameters {Hyperparameters} -- A configuration for hyperparameters used for training.
 
         Returns:
-            Network -- Reconstructed network object.
+            Network: Reconstructed network object.
         """
         network = Network.get_from_model(model_filepath, hyperparameters)
         network.__model.load_weights(weights_filepath)
         return network
 
     @property
-    def input_shape(self) -> Tuple:
+    def input_shape(self) -> tuple:
         """Get the input shape of the network.
 
         Returns:
-            tuple -- The input shape of the network.
+            tuple: The input shape of the network.
         """
 
         return self.__input_shape
@@ -182,7 +206,7 @@ class Network(object):
         """Get the type of the network.
 
         Returns:
-            string -- The type of the network.
+            str: The type of the network.
         """
 
         return self.__n_type
@@ -195,11 +219,11 @@ class Network(object):
         self.__model.summary()
 
     @property
-    def layers(self) -> List[Any]:
+    def layers(self) -> list:
         """Get the layers of the network.
 
         Returns:
-            list -- The statck of layers contained by the network
+            list: The stack of layers contained by the network
         """
 
         return self.__model.layers
@@ -212,7 +236,7 @@ class Network(object):
             weights_filepath {string} -- The weights file path.
 
         Returns:
-            numpy.ndarray -- The Numpy array of predictions.
+            numpy.ndarray: The Numpy array of predictions.
         """
         self.__model.load_weights(weights_filepath)
         return self.__model.predict_on_batch(input_data)
@@ -237,8 +261,12 @@ class Network(object):
             logs_dir {string} -- The TensorBoard log file directory.
             training_log {string} -- The path to the log file of epoch results.
             resume {bool} -- True to continue with previous training result or False to start a new one (default: {False}).
+
         Returns:
-            tuple -- A tuple contains validation losses and validation accuracies.
+            tuple: A tuple contains validation losses and validation accuracies.
+
+        Raises:
+            TerminalException: If the predication is interrupted by user hitting the interrupt key
         """
 
         csv_logger = (
@@ -297,11 +325,11 @@ class Network(object):
                 initial_epoch=initial_epoch,
             )
         except KeyboardInterrupt:
-            Network.__LOGGER.warning("Training interrupted by the user")
+            self.__LOGGER.warning("Training interrupted by the user")
             raise TerminalException("Training interrupted by the user")
         finally:
             save_model(self.__model, model_filepath)
-            Network.__LOGGER.warning("Model saved to %s" % model_filepath)
+            self.__LOGGER.warning("Model saved to %s" % model_filepath)
 
         return hist.history["val_loss"], hist.history["val_acc"] if int(tf.__version__.split(".")[0]) < 2 else hist.history["val_accuracy"]
 
@@ -325,8 +353,12 @@ class Network(object):
             logs_dir {string} -- The TensorBoard log file directory.
             training_log {string} -- The path to the log file of epoch results.
             resume {bool} -- True to continue with previous training result or False to start a new one (default: {False}).
+
         Returns:
-            tuple -- A tuple contains validation losses and validation accuracies.
+            tuple: A tuple contains validation losses and validation accuracies.
+
+        Raises:
+            TerminalException: If the training is interrupted by user hitting the interrupt key
         """
 
         initial_epoch = 0
@@ -392,11 +424,11 @@ class Network(object):
                 initial_epoch=initial_epoch,
             )
         except KeyboardInterrupt:
-            Network.__LOGGER.warning("Training interrupted by the user")
+            self.__LOGGER.warning("Training interrupted by the user")
             raise TerminalException("Training interrupted by the user")
         finally:
             self.__model.save(model_filepath)
-            Network.__LOGGER.warning("Model saved to %s" % model_filepath)
+            self.__LOGGER.warning("Model saved to %s" % model_filepath)
 
         return hist.history["val_loss"], hist.history["val_acc"] if int(tf.__version__.split(".")[0]) < 2 else hist.history["val_accuracy"]
 
@@ -417,7 +449,7 @@ class Network(object):
             hyperparameters {Hyperparameters} -- A configuration for hyperparameters used for training.
 
         Returns:
-            tuple -- A tuple contains validation losses and validation accuracies.
+            tuple: A tuple contains validation losses and validation accuracies.
         """
 
         network = cls(cls.__secret, input_shape, hyperparameters)
@@ -456,8 +488,9 @@ class Network(object):
             train_data_raw {list} -- The HDF5 raw training data.
             labels_raw {list} -- The HDF5 raw training labels.
             hyperparameters {Hyperparameters} -- A configuration for hyperparameters used for training.
+
         Returns:
-            tuple -- A tuple contains validation losses and validation accuracies.
+            tuple: A tuple contains validation losses and validation accuracies.
         """
 
         network = cls(cls.__secret, input_shape, hyperparameters)
@@ -548,14 +581,14 @@ class Network(object):
             # Set the number of inter/intra threads to the number of physical cores (experiment shows this is the best)
             physical_core_num = psutil.cpu_count(logical=False)
             tf.config.threading.set_inter_op_parallelism_threads(physical_core_num)
-            tf.config.threading.set_intra_op_parallelism_threads(physical_core_num)
+            tf.config.threading.set_intra_op_parallelism_threads(1)
             tf.config.set_soft_device_placement(True)
             physical_devices = tf.config.experimental.list_physical_devices("GPU")
             try:
                 for gpu in physical_devices:
                     tf.config.experimental.set_memory_growth(gpu, True)
             except Exception:
-                Network.__LOGGER.warning("Invalid device or cannot modify virtual devices once initialised")
+                raise ValueError("Invalid device or cannot modify virtual devices once initialised")
             K.clear_session()
         elif backend.lower() == "theano" or backend.lower() == "cntk":
             #  Backends other than tensorflow require separate installations before being used.
