@@ -6,25 +6,37 @@ import concurrent.futures
 import gc
 import math
 import logging
+import tempfile
+import librosa
 import numpy as np
 import multiprocessing as mp
+import soundfile as sf
 from typing import Tuple, List, Optional, Dict, Any, Iterable, Union
+from copy import deepcopy
 from pysrt import SubRipTime, SubRipItem, SubRipFile
 from sklearn.metrics import log_loss
-from copy import deepcopy
 from .network import Network
 from .embedder import FeatureEmbedder
 from .media_helper import MediaHelper
-from .singleton import Singleton
 from .subtitle import Subtitle
 from .hyperparameters import Hyperparameters
-from .exception import TerminalException
-from .exception import NoFrameRateException
+from .lib.language import Language
+from .utils import Utils
+from .exception import TerminalException, NoFrameRateException
 from .logger import Logger
 
 
-class Predictor(metaclass=Singleton):
+class Predictor(object):
     """ Predictor for working out the time to shift subtitles
+
+    Keyword Arguments:
+        media_process_timeout {int} -- The maximum waiting time in seconds when processing media files.
+        segment_alignment_timeout {int} -- The maximum waiting time in seconds when aligning each segment.
+        n_mfcc {int} -- The number of MFCC components (default: {13}).
+        frequency {float} -- The sample rate  (default: {16000}).
+        hop_len {int} -- The number of samples per frame (default: {512}).
+        step_sample {float} -- The space (in seconds) between the begining of each sample (default: 1s / 25 FPS = 0.04s).
+        len_sample {float} -- The length in seconds for the input samples (default: {0.075}).
     """
     __MAX_SHIFT_IN_SECS = (
         100
@@ -34,25 +46,14 @@ class Predictor(metaclass=Singleton):
     )  # Average 0.3 word per sec multiplies average 6 characters per word
     __MAX_HEAD_ROOM = 20000  # Maximum duration without subtitle (10 minutes)
 
-    __SEGMENT_PREDICTION_TIMEOUT = 60  # Maximum waiting time in seconds when predicting each segment
-
     __THREAD_QUEUE_SIZE = 8
     __THREAD_NUMBER = 1  # Do not change
 
-    def __init__(self, **kwargs) -> None:
-        """Feature predictor initialiser.
-
-            Keyword Arguments:
-                n_mfcc {int} -- The number of MFCC components (default: {13}).
-                frequency {float} -- The sample rate  (default: {16000}).
-                hop_len {int} -- The number of samples per frame (default: {512}).
-                step_sample {float} -- The space (in seconds) between the begining of each sample (default: 1s / 25 FPS = 0.04s).
-                len_sample {float} -- The length in seconds for the input samples (default: {0.075}).
-        """
-
+    def __init__(self, media_process_timeout: int = 180, segment_alignment_timeout: int = 60, **kwargs) -> None:
+        self.__media_helper = MediaHelper(media_process_timeout=media_process_timeout)
+        self.__segment_alignment_timeout = segment_alignment_timeout
         self.__feature_embedder = FeatureEmbedder(**kwargs)
         self.__LOGGER = Logger().get_logger(__name__)
-        self.__media_helper = MediaHelper()
 
     def predict_single_pass(
             self,
@@ -68,7 +69,7 @@ class Predictor(metaclass=Singleton):
                 weights_dir {string} -- The the model weights directory.
 
             Returns:
-                tuple -- The shifted subtitles, the audio file path and the voice probabilities of the original audio.
+                tuple: The shifted subtitles, the audio file path and the voice probabilities of the original audio.
         """
 
         weights_file_path = self.__get_weights_path(weights_dir)
@@ -101,15 +102,15 @@ class Predictor(metaclass=Singleton):
         """Predict time to shift with single pass
 
             Arguments:
-            video_file_path {string} -- The input video file path.
-            subtitle_file_path {string} -- The path to the subtitle file.
-            weights_dir {string} -- The the model weights directory.
-            stretch {bool} -- True to stretch the subtitle segments (default: {False})
-            stretch_in_lang {str} -- The language used for stretching subtitles (default: {"eng"}).
-            exit_segfail {bool} -- True to exit on any segment alignment failures (default: {False})
+                video_file_path {string} -- The input video file path.
+                subtitle_file_path {string} -- The path to the subtitle file.
+                weights_dir {string} -- The the model weights directory.
+                stretch {bool} -- True to stretch the subtitle segments (default: {False})
+                stretch_in_lang {str} -- The language used for stretching subtitles (default: {"eng"}).
+                exit_segfail {bool} -- True to exit on any segment alignment failures (default: {False})
 
             Returns:
-            tuple -- The shifted subtitles, the globally shifted subtitles and the voice probabilities of the original audio.
+                tuple: The shifted subtitles, the globally shifted subtitles and the voice probabilities of the original audio.
         """
 
         weights_file_path = self.__get_weights_path(weights_dir)
@@ -139,16 +140,24 @@ class Predictor(metaclass=Singleton):
             if os.path.exists(audio_file_path):
                 os.remove(audio_file_path)
 
-    def predict_plain_text(self, video_file_path: str, subtitle_file_path: str, stretch_in_lang: str = "eng") -> Tuple:
+    def predict_plain_text(self,
+                           video_file_path: str,
+                           subtitle_file_path: str,
+                           stretch_in_lang: str = "eng",
+                           with_word_time_codes: bool = False) -> tuple:
         """Predict time to shift with plain texts
 
             Arguments:
-            video_file_path {string} -- The input video file path.
-            subtitle_file_path {string} -- The path to the subtitle file.
-            stretch_in_lang {str} -- The language used for stretching subtitles (default: {"eng"}).
+                video_file_path {string} -- The input video file path.
+                subtitle_file_path {string} -- The path to the subtitle file.
+                stretch_in_lang {str} -- The language used for stretching subtitles (default: {"eng"}).
+                with_word_time_codes {bool} -- True to output time codes for each word (default: {False}).
 
             Returns:
-            tuple -- The shifted subtitles, the audio file path (None) and the voice probabilities of the original audio (None).
+                tuple: The shifted subtitles, the audio file path (None) and the voice probabilities of the original audio (None).
+
+            Raises:
+                TerminalException: If the predication is interrupted by user hitting the interrupt key.
         """
         from aeneas.executetask import ExecuteTask
         from aeneas.task import Task
@@ -174,9 +183,22 @@ class Predictor(metaclass=Singleton):
         runtime_config_string = "dtw_algorithm=stripe"  # stripe or exact
         task = Task(config_string=task_config_string)
 
+        path = None
+        if with_word_time_codes:
+            _, path = tempfile.mkstemp()
+            processed = []
+            with open(subtitle_file_path, "r", encoding="utf-8") as f:
+                for line in f.readlines():
+                    # TODO: Use tokenizers to process languages that do not use spaces as word delimiters
+                    processed.append((os.linesep * 2).join(line.strip().split()) if line.strip() else os.linesep)
+
+            with open(path, "w", encoding="utf-8") as f:
+                f.write((os.linesep * 2).join(processed))
+                f.flush()
+
         try:
             task.audio_file_path_absolute = audio_file_path
-            task.text_file_path_absolute = subtitle_file_path
+            task.text_file_path_absolute = subtitle_file_path if not with_word_time_codes else path
             task.sync_map_file_path_absolute = "{}.srt".format(root)
 
             tee = False if self.__LOGGER.level == getattr(logging, 'DEBUG') else True
@@ -192,15 +214,17 @@ class Predictor(metaclass=Singleton):
             task.output_sync_map_file()
 
             # Load the above subtitle segment
-            adjusted_subs = Subtitle.load(
-                task.sync_map_file_path_absolute
-            ).subs
+            adjusted_subs = Subtitle._get_srt_subs(
+                subrip_file_path=task.sync_map_file_path_absolute,
+                encoding="utf-8"
+            )
 
             frame_rate = None
             try:
                 frame_rate = self.__media_helper.get_frame_rate(video_file_path)
                 self.__feature_embedder.step_sample = 1 / frame_rate
-                self.__on_frame_timecodes(adjusted_subs)
+                if not with_word_time_codes:
+                    self.__on_frame_timecodes(adjusted_subs)
             except NoFrameRateException:
                 self.__LOGGER.warning("Cannot detect the frame rate for %s" % video_file_path)
 
@@ -215,6 +239,8 @@ class Predictor(metaclass=Singleton):
                 os.remove(task.audio_file_path_absolute)
             if task.sync_map_file_path_absolute is not None and os.path.exists(task.sync_map_file_path_absolute):
                 os.remove(task.sync_map_file_path_absolute)
+            if path is not None and os.path.exists(path):
+                os.remove(path)
 
     def get_log_loss(self, voice_probabilities: np.ndarray, subs: List[SubRipItem]) -> float:
         """Returns a single loss value on voice prediction
@@ -223,8 +249,11 @@ class Predictor(metaclass=Singleton):
                 voice_probabilities {list} -- A list of probabilities of audio chunks being speech.
                 subs {list} -- A list of subtitle segments.
 
-                Returns:
-                    float -- The loss value.
+            Returns:
+                float: The loss value.
+
+            Raises:
+                TerminalException: If the subtitle mask is empty.
         """
 
         subtitle_mask = Predictor.__get_subtitle_mask(self, subs)
@@ -258,8 +287,12 @@ class Predictor(metaclass=Singleton):
             Arguments:
                 voice_probabilities {list} -- A list of probabilities of audio chunks being speech.
                 subs {list} -- A list of subtitle segments.
+
             Returns:
-                tuple -- The minimum loss value and its position.
+                tuple: The minimum loss value and its position.
+
+            Raises:
+                TerminalException: If subtitle is empty or suspicious audio/subtitle duration is detected.
         """
 
         local_subs = deepcopy(subs)
@@ -357,14 +390,14 @@ class Predictor(metaclass=Singleton):
                 )
             for i, future in enumerate(futures):
                 try:
-                    new_subs = future.result(timeout=Predictor.__SEGMENT_PREDICTION_TIMEOUT)
+                    new_subs = future.result(timeout=self.__segment_alignment_timeout)
                 except concurrent.futures.TimeoutError as e:
-                    self.__cancel_futures(futures[i:], Predictor.__SEGMENT_PREDICTION_TIMEOUT)
-                    message = "Segment alignment timed out after {} seconds".format(Predictor.__SEGMENT_PREDICTION_TIMEOUT)
+                    self.__cancel_futures(futures[i:], self.__segment_alignment_timeout)
+                    message = "Segment alignment timed out after {} seconds".format(self.__segment_alignment_timeout)
                     self.__LOGGER.error(message)
                     raise TerminalException(message) from e
                 except Exception as e:
-                    self.__cancel_futures(futures[i:], Predictor.__SEGMENT_PREDICTION_TIMEOUT)
+                    self.__cancel_futures(futures[i:], self.__segment_alignment_timeout)
                     message = "Exception on segment alignment: {}\n{}".format(str(e), "".join(traceback.format_stack()))
                     self.__LOGGER.error(e, exc_info=True, stack_info=True)
                     traceback.print_tb(e.__traceback__)
@@ -373,7 +406,7 @@ class Predictor(metaclass=Singleton):
                     else:
                         raise TerminalException(message) from e
                 except KeyboardInterrupt:
-                    self.__cancel_futures(futures[i:], Predictor.__SEGMENT_PREDICTION_TIMEOUT)
+                    self.__cancel_futures(futures[i:], self.__segment_alignment_timeout)
                     raise TerminalException("Segment alignment interrupted by the user")
                 else:
                     self.__LOGGER.debug("Segment aligned")
@@ -437,7 +470,7 @@ class Predictor(metaclass=Singleton):
             gc.collect()
 
             if stretch:
-                subs_new = self.__adjust_durations(subs_new, audio_file_path, stretch_in_lang, lock)
+                subs_new = self.__compress_and_stretch(subs_new, audio_file_path, stretch_in_lang, lock)
                 self.__LOGGER.info("[{}] Segment {} stretched".format(os.getpid(), segment_index))
             return subs_new
         except Exception as e:
@@ -522,10 +555,17 @@ class Predictor(metaclass=Singleton):
         Arguments:
             audio_file_path {string} -- The file path of the original audio.
             subs {list} -- A list of SubRip files.
-            weights_file_path {string} --  The file path of the weights file.
+            weights_file_path {string}:  The file path of the weights file.
             stretch {bool} -- True to stretch the subtitle segments.
             stretch_in_lang {str} -- The language used for stretching subtitles.
             exit_segfail {bool} -- True to exit on any segment alignment failures.
+
+        Returns:
+            list: A list of aligned SubRip files
+
+        Raises:
+            TerminalException: If the alignment is interrupted by user hitting the interrupt key or times out
+            Exception: Thrown when any other exceptions occur.
         """
 
         segment_starts, segment_ends, subs = self.__media_helper.get_audio_segment_starts_and_ends(subs)
@@ -573,14 +613,14 @@ class Predictor(metaclass=Singleton):
             ]
             for i, future in enumerate(futures):
                 try:
-                    subs_list.extend(future.result(timeout=Predictor.__SEGMENT_PREDICTION_TIMEOUT * batch_size))
+                    subs_list.extend(future.result(timeout=self.__segment_alignment_timeout * batch_size))
                 except concurrent.futures.TimeoutError as e:
-                    self.__cancel_futures(futures[i:], Predictor.__SEGMENT_PREDICTION_TIMEOUT * batch_size)
-                    message = "Batch alignment timed out after {} seconds".format(Predictor.__SEGMENT_PREDICTION_TIMEOUT)
+                    self.__cancel_futures(futures[i:], self.__segment_alignment_timeout * batch_size)
+                    message = "Batch alignment timed out after {} seconds".format(self.__segment_alignment_timeout)
                     self.__LOGGER.error(message)
                     raise TerminalException(message) from e
                 except Exception as e:
-                    self.__cancel_futures(futures[i:], Predictor.__SEGMENT_PREDICTION_TIMEOUT * batch_size)
+                    self.__cancel_futures(futures[i:], self.__segment_alignment_timeout * batch_size)
                     message = "Exception on batch alignment: {}\n{}".format(str(e), "".join(traceback.format_stack()))
                     self.__LOGGER.error(e, exc_info=True, stack_info=True)
                     traceback.print_tb(e.__traceback__)
@@ -589,7 +629,7 @@ class Predictor(metaclass=Singleton):
                     else:
                         raise TerminalException(message) from e
                 except KeyboardInterrupt:
-                    self.__cancel_futures(futures[i:], Predictor.__SEGMENT_PREDICTION_TIMEOUT * batch_size)
+                    self.__cancel_futures(futures[i:], self.__segment_alignment_timeout * batch_size)
                     raise TerminalException("Batch alignment interrupted by the user")
                 else:
                     self.__LOGGER.debug("Batch aligned")
@@ -672,9 +712,10 @@ class Predictor(metaclass=Singleton):
                 task.output_sync_map_file()
 
             # Load the above subtitle segment
-            adjusted_subs = Subtitle.load(
-                task.sync_map_file_path_absolute
-            ).subs
+            adjusted_subs = Subtitle._get_srt_subs(
+                subrip_file_path=task.sync_map_file_path_absolute,
+                encoding="utf-8"
+            )
             for index, sub_new_loaded in enumerate(adjusted_subs):
                 sub_new_loaded.index = subs[index].index
 
@@ -699,6 +740,111 @@ class Predictor(metaclass=Singleton):
             if task.sync_map_file_path_absolute is not None and os.path.exists(task.sync_map_file_path_absolute):
                 os.remove(task.sync_map_file_path_absolute)
 
+    def __compress_and_stretch(self, subs: List[SubRipItem], audio_file_path: str, stretch_in_lang: str, lock: threading.RLock) -> List[SubRipItem]:
+        from dtw import dtw
+        try:
+            with lock:
+                segment_path, _ = self.__media_helper.extract_audio_from_start_to_end(
+                    audio_file_path,
+                    str(subs[0].start),
+                    str(subs[len(subs) - 1].end),
+                )
+
+                # Create a text file for DTW alignments
+                root, _ = os.path.splitext(segment_path)
+                text_file_path = "{}.txt".format(root)
+
+                with open(text_file_path, "w", encoding="utf8") as text_file:
+                    text_file.write("*****".join([sub_new.text for sub_new in subs]))
+
+            sample_rate = self.__feature_embedder.frequency
+            hop_length = self.__feature_embedder.hop_len
+            n_mfcc = self.__feature_embedder.n_mfcc
+
+            file_script_duration_mapping = []
+            with tempfile.TemporaryDirectory() as temp_dir:
+                with open(text_file_path, "r") as f:
+                    script_lines = f.read().split("*****")
+                    wav_data = []
+                    for i, line in enumerate(script_lines):
+                        normalised_line = line.replace('"', "'")
+                        espeak_output_file = f"espeak_part_{i}.wav"
+                        espeak_cmd = f"espeak -v {Language.LANGUAGE_TO_VOICE_CODE[stretch_in_lang]} --stdout -- \"{normalised_line}\" | ffmpeg -y -i - -af 'aresample={sample_rate}' {os.path.join(temp_dir, espeak_output_file)}"
+                        os.system(espeak_cmd)
+                        y, sr = librosa.load(os.path.join(temp_dir, espeak_output_file), sr=None)
+                        wav_data.append(y)
+                        duration = librosa.get_duration(y=y, sr=sr)
+                        file_script_duration_mapping.append((os.path.join(temp_dir, espeak_output_file), line, duration))
+                    data = np.concatenate(wav_data)
+                    sf.write(os.path.join(temp_dir, "espeak-all.wav"), data, sr)
+
+                y_query, sr_query = librosa.load(os.path.join(temp_dir, "espeak-all.wav"), sr=None)
+                query_mfcc_features = librosa.feature.mfcc(y=y_query, sr=sr_query, n_mfcc=n_mfcc, hop_length=hop_length).T
+                y_reference, sr_reference = librosa.load(segment_path, sr=sample_rate)
+                reference_mfcc_features = librosa.feature.mfcc(y=y_reference, sr=sr_reference, n_mfcc=n_mfcc, hop_length=hop_length).T
+
+                alignment = dtw(query_mfcc_features, reference_mfcc_features, keep_internals=False)
+                assert len(alignment.index1) == len(alignment.index2), "Mismatch in lengths of alignment indices"
+                assert sr_query == sr_reference
+                frame_duration = hop_length / sr_query
+
+                mapped_times = []
+                start_frame_index = 0
+                for index, (wav_file, line_text, duration) in enumerate(file_script_duration_mapping):
+                    num_frames_in_query = int(np.ceil(duration / frame_duration))
+
+                    query_start_frame = start_frame_index
+                    query_end_frame = start_frame_index + num_frames_in_query - 1
+                    reference_frame_indices = [r for q, r in zip(alignment.index1, alignment.index2) if
+                                               query_start_frame <= q <= query_end_frame]
+                    reference_start_frame = min(reference_frame_indices)
+                    reference_end_frame = max(reference_frame_indices)
+
+                    # TODO: Handle cases where mapped frames are not found in the reference audio
+
+                    new_reference_start_time = reference_start_frame * frame_duration
+                    new_reference_end_time = (reference_end_frame + 1) * frame_duration
+
+                    mapped_times.append({
+                        "new_reference_start_time": new_reference_start_time,
+                        "new_reference_end_time": new_reference_end_time
+                    })
+
+                    start_frame_index = query_end_frame + 1
+
+                with open(os.path.join(temp_dir, "synced_subtitles.srt"), "w") as f:
+                    for index, entry in enumerate(mapped_times):
+                        start_srt = Utils.format_timestamp(entry["new_reference_start_time"])
+                        end_srt = Utils.format_timestamp(entry["new_reference_end_time"])
+                        f.write(f"{index + 1}\n")
+                        f.write(f"{start_srt} --> {end_srt}\n")
+                        f.write(f"{script_lines[index]}\n")
+                        f.write(f"\n")
+                    f.flush()
+
+                adjusted_subs = Subtitle._get_srt_subs(
+                    subrip_file_path=os.path.join(temp_dir, "synced_subtitles.srt"),
+                    encoding="utf-8"
+                )
+
+            for index, sub_new_loaded in enumerate(adjusted_subs):
+                sub_new_loaded.index = subs[index].index
+
+            adjusted_subs.shift(
+                seconds=self.__media_helper.get_duration_in_seconds(
+                    start=None, end=str(subs[0].start)
+                )
+            )
+            return adjusted_subs
+        except KeyboardInterrupt:
+            raise TerminalException("Subtitle compress and stretch interrupted by the user")
+        finally:
+            # Housekeep intermediate files
+            if text_file_path is not None and os.path.exists(
+                    text_file_path
+            ):
+                os.remove(text_file_path)
+
     def __predict(
             self,
             video_file_path: Optional[str],
@@ -725,7 +871,11 @@ class Predictor(metaclass=Singleton):
             previous_gap {float} -- The duration between the start time of the audio segment and the start time of the subtitle segment (default: {None}).
 
         Returns:
-            tuple -- The shifted subtitles, the audio file path and the voice probabilities of the original audio.
+            tuple: The shifted subtitles, the audio file path and the voice probabilities of the original audio.
+
+        Raises:
+            TerminalException: If the prediction failed on invalid input or on other exceptions.
+            ValueError: Thrown when no subtitle is passed in.
         """
         if network is None:
             network = self.__initialise_network(os.path.dirname(weights_file_path), self.__LOGGER)
