@@ -13,8 +13,15 @@ from .exception import UnsupportedFormatException
 
 class Subtitle(object):
     """Load a subtitle file into internal data structure
-    """
 
+    Arguments:
+        secret {object} -- A hash only known by factory methods.
+        subtitle_file_path {string} -- The path to the subtitle file.
+        format {string} -- Supported subtitle formats: subrip and ttml.
+
+    Raises:
+        UnsupportedFormatException: Thrown when the input subtitle format is not supported or no subtitle content is found.
+    """
     __secret = object()
 
     ElementTree.register_namespace("", "http://www.w3.org/ns/ttml")
@@ -38,19 +45,9 @@ class Subtitle(object):
     SCC_EXTENSIONS = [".scc"]
     SBV_EXTENSIONS = [".sbv"]
     YT_TRANSCRIPT_EXTENSIONS = [".ytt"]
+    JSON_RAW_EXTENSIONS = [".json"]
 
     def __init__(self, secret: object, subtitle_file_path: str, subtitle_format: str) -> None:
-        """Subtitle object initialiser.
-
-        Arguments:
-            secret {object} -- A hash only known by factory methods.
-            subtitle_file_path {string} -- The path to the subtitle file.
-            format {string} -- Supported subtitle formats: subrip and ttml.
-
-        Raises:
-            NotImplementedError --  Thrown when any subtitle attributes are modified.
-        """
-
         assert (
             secret == Subtitle.__secret
         ), "Only factory methods are supported when creating instances"
@@ -85,6 +82,8 @@ class Subtitle(object):
             self.__subs = self.__convert_sbv_to_subs(subtitle_file_path)
         elif subtitle_format == "ytt":
             self.__subs = self.__convert_ytt_to_subs(subtitle_file_path)
+        elif subtitle_format == "json":
+            self.__subs = self.__convert_json_raw_to_subs(subtitle_file_path)
         else:
             raise UnsupportedFormatException(
                 "Unknown subtitle format for file: {}".format(subtitle_file_path)
@@ -102,7 +101,7 @@ class Subtitle(object):
             subtitle_file_path {string} -- The path to the subtitle file.
 
         Returns:
-            Subtitle -- Subtitle object.
+            Subtitle: Subtitle object.
         """
 
         return cls(cls.__secret, subtitle_file_path, "subrip")
@@ -115,7 +114,7 @@ class Subtitle(object):
             subrip_str {string} -- The string representation of the SubRip content.
 
         Returns:
-            Subtitle -- Subtitle object.
+            Subtitle: Subtitle object.
         """
 
         return cls(cls.__secret, subrip_raw, "subrip_raw")
@@ -128,7 +127,7 @@ class Subtitle(object):
             subtitle_file_path {string} -- The path to the subtitle file.
 
         Returns:
-            Subtitle -- Subtitle object.
+            Subtitle: Subtitle object.
         """
 
         return cls(cls.__secret, subtitle_file_path, "ttml")
@@ -141,7 +140,7 @@ class Subtitle(object):
             subtitle_file_path {string} -- The path to the subtitle file.
 
         Returns:
-            Subtitle -- Subtitle object.
+            Subtitle: Subtitle object.
         """
 
         return cls(cls.__secret, subtitle_file_path, "webvtt")
@@ -154,7 +153,7 @@ class Subtitle(object):
             subtitle_file_path {string} -- The path to the subtitle file.
 
         Returns:
-            Subtitle -- Subtitle object.
+            Subtitle: Subtitle object.
         """
 
         return cls(cls.__secret, subtitle_file_path, "ssa")
@@ -167,7 +166,7 @@ class Subtitle(object):
             subtitle_file_path {string} -- The path to the subtitle file.
 
         Returns:
-            Subtitle -- Subtitle object.
+            Subtitle: Subtitle object.
         """
 
         return cls(cls.__secret, subtitle_file_path, "ass")
@@ -180,7 +179,7 @@ class Subtitle(object):
             subtitle_file_path {string} -- The path to the subtitle file.
 
         Returns:
-            Subtitle -- Subtitle object.
+            Subtitle: Subtitle object.
         """
 
         return cls(cls.__secret, subtitle_file_path, "microdvd")
@@ -193,7 +192,7 @@ class Subtitle(object):
             subtitle_file_path {string} -- The path to the subtitle file.
 
         Returns:
-            Subtitle -- Subtitle object.
+            Subtitle: Subtitle object.
         """
 
         return cls(cls.__secret, subtitle_file_path, "mpl2")
@@ -206,7 +205,7 @@ class Subtitle(object):
             subtitle_file_path {string} -- The path to the subtitle file.
 
         Returns:
-            Subtitle -- Subtitle object.
+            Subtitle: Subtitle object.
         """
 
         return cls(cls.__secret, subtitle_file_path, "tmp")
@@ -219,7 +218,7 @@ class Subtitle(object):
             subtitle_file_path {string} -- The path to the subtitle file.
 
         Returns:
-            Subtitle -- Subtitle object.
+            Subtitle: Subtitle object.
         """
 
         return cls(cls.__secret, subtitle_file_path, "sami")
@@ -232,7 +231,7 @@ class Subtitle(object):
             subtitle_file_path {string} -- The path to the subtitle file.
 
         Returns:
-            Subtitle -- Subtitle object.
+            Subtitle: Subtitle object.
         """
 
         return cls(cls.__secret, subtitle_file_path, "stl")
@@ -245,7 +244,7 @@ class Subtitle(object):
             subtitle_file_path {string} -- The path to the subtitle file.
 
         Returns:
-            Subtitle -- Subtitle object.
+            Subtitle: Subtitle object.
         """
 
         return cls(cls.__secret, subtitle_file_path, "scc")
@@ -258,7 +257,7 @@ class Subtitle(object):
             subtitle_file_path {string} -- The path to the subtitle file.
 
         Returns:
-            Subtitle -- Subtitle object.
+            Subtitle: Subtitle object.
         """
 
         return cls(cls.__secret, subtitle_file_path, "sbv")
@@ -271,11 +270,24 @@ class Subtitle(object):
             subtitle_file_path {string} -- The path to the subtitle file.
 
         Returns:
-            Subtitle -- Subtitle object.
+            Subtitle: Subtitle object.
         """
 
         return cls(cls.__secret, subtitle_file_path, "ytt")
 
+    @classmethod
+    def load_json(cls, subtitle_file_path: str) -> "Subtitle":
+        """Load a JSON raw subtitle file.
+
+        Arguments:
+            subtitle_file_path {string} -- The path to the subtitle file.
+
+        Returns:
+            Subtitle: Subtitle object.
+        """
+
+        return cls(cls.__secret, subtitle_file_path, "json")
+
     @classmethod
     def load(cls, subtitle_file_path: str) -> "Subtitle":
         """Load a SubRip or TTML subtitle file based on the file extension.
@@ -284,7 +296,7 @@ class Subtitle(object):
             subtitle_file_path {string} -- The path to the subtitle file.
 
         Returns:
-            Subtitle -- Subtitle object.
+            Subtitle: Subtitle object.
         """
 
         _, file_extension = os.path.splitext(subtitle_file_path.lower())
@@ -314,6 +326,8 @@ class Subtitle(object):
             return cls(cls.__secret, subtitle_file_path, "sbv")
         elif file_extension in cls.YT_TRANSCRIPT_EXTENSIONS:
             return cls(cls.__secret, subtitle_file_path, "ytt")
+        elif file_extension in cls.JSON_RAW_EXTENSIONS:
+            return cls(cls.__secret, subtitle_file_path, "json")
         else:
             return cls(cls.__secret, subtitle_file_path, "unknown")
 
@@ -336,7 +350,10 @@ class Subtitle(object):
             suffix {string} -- The suffix used as part of the aligned subtitle file name.
 
         Returns:
-            string -- The path to the shifted subtitle file.
+            string: The path to the shifted subtitle file.
+
+        Raises:
+            UnsupportedFormatException: Thrown when the input subtitle format is not supported.
         """
         _, file_extension = os.path.splitext(subtitle_file_path)
         if shifted_subtitle_file_path is None:
@@ -381,6 +398,8 @@ class Subtitle(object):
                 subs = cls(cls.__secret, subtitle_file_path, "sbv").subs
             elif file_extension.lower() in cls.YT_TRANSCRIPT_EXTENSIONS:
                 subs = cls(cls.__secret, subtitle_file_path, "ytt").subs
+            elif file_extension.lower() in cls.JSON_RAW_EXTENSIONS:
+                subs = cls(cls.__secret, subtitle_file_path, "json").subs
             else:
                 raise UnsupportedFormatException(
                     "Unknown subtitle format for file: {}".format(subtitle_file_path)
@@ -430,7 +449,7 @@ class Subtitle(object):
             se_uppercase {bool} -- True when the sound effect is in uppercase or False when in lowercase (default: {True}).
 
         Returns:
-            {list} -- A list of SubRipItems.
+            list: A list of SubRipItems.
         """
         new_subs = deepcopy(subs)
         for sub in subs:
@@ -451,7 +470,7 @@ class Subtitle(object):
             se_suffix {string} -- A suffix indicating the end of the sound effect (default: {None}).
 
         Returns:
-            {list} -- A list of SubRipItems.
+            list: A list of SubRipItems.
         """
         new_subs = deepcopy(subs)
         for sub in subs:
@@ -476,7 +495,7 @@ class Subtitle(object):
             subtitle_file_path {string} -- The path to the subtitle file.
 
         Returns:
-            {string} -- The plain text of subtitle.
+            str: The plain text of subtitle.
         """
 
         subs = Subtitle.load(subtitle_file_path).subs
@@ -488,13 +507,13 @@ class Subtitle(object):
         """Get the file extensions of the supported subtitles.
 
         Returns:
-            {set} -- The subtitle extensions.
+            set: The subtitle extensions.
         """
         return set(Subtitle.SUBRIP_EXTENTIONS + Subtitle.TTML_EXTENSIONS + Subtitle.WEBVTT_EXTENSIONS
                    + Subtitle.SSA_EXTENTIONS + Subtitle.ADVANCED_SSA_EXTENTIONS + Subtitle.MICRODVD_EXTENSIONS
                    + Subtitle.MPL2_EXTENSIONS + Subtitle.TMP_EXTENSIONS + Subtitle.SAMI_EXTENSIONS
                    + Subtitle.STL_EXTENSIONS + Subtitle.SCC_EXTENSIONS + Subtitle.SBV_EXTENSIONS
-                   + Subtitle.YT_TRANSCRIPT_EXTENSIONS)
+                   + Subtitle.YT_TRANSCRIPT_EXTENSIONS + Subtitle.JSON_RAW_EXTENSIONS)
 
     @property
     def subtitle_file_path(self) -> str:
@@ -508,13 +527,13 @@ class Subtitle(object):
     def __load_subrip(subrip_file_path: str) -> SubRipFile:
         """Load a subtitle file in the SubRip format
 
-                Arguments:
-                    subrip_file_path {string} -- The path to the SubRip subtitle file.
+        Arguments:
+            subrip_file_path {string} -- The path to the SubRip subtitle file.
 
-                Returns:
-                    {list} -- A list of SubRipItems.
-                """
-        return Subtitle.__get_srt_subs(subrip_file_path)
+        Returns:
+            SubRipFile: A list of SubRipItems.
+        """
+        return Subtitle._get_srt_subs(subrip_file_path)
 
     @staticmethod
     def __convert_ttml_to_subs(ttml_file_path: str) -> SubRipFile:
@@ -524,13 +543,13 @@ class Subtitle(object):
             ttml_file_path {string} -- The path to the TTML subtitle file.
 
         Returns:
-            {list} -- A list of SubRipItems.
+            SubRipFile: A list of SubRipItems.
         """
 
         _, path = tempfile.mkstemp()
         Utils.ttml2srt(ttml_file_path, path)
 
-        return Subtitle.__get_srt_subs(path, housekeep=True)
+        return Subtitle._get_srt_subs(path, housekeep=True)
 
     @staticmethod
     def __convert_vtt_to_subs(vtt_file_path: str) -> SubRipFile:
@@ -540,13 +559,13 @@ class Subtitle(object):
             vtt_file_path {string} -- The path to the WebVTT subtitle file.
 
         Returns:
-            {list} -- A list of SubRipItems.
+            SubRipFile: A list of SubRipItems.
         """
 
         _, path = tempfile.mkstemp()
         Utils.vtt2srt(vtt_file_path, path)
 
-        return Subtitle.__get_srt_subs(path, housekeep=True)
+        return Subtitle._get_srt_subs(path, housekeep=True)
 
     @staticmethod
     def __convert_ssa_to_subs(ssa_file_path: str) -> SubRipFile:
@@ -556,14 +575,14 @@ class Subtitle(object):
             ass_file_path {string} -- The path to the SubStation Alpha v4.0 subtitle file.
 
         Returns:
-            {list} -- A list of SubRipItems.
+            SubRipFile: A list of SubRipItems.
         """
 
         _, path = tempfile.mkstemp()
         path = "%s.srt" % path
         Utils.ssa2srt(ssa_file_path, path)
 
-        return Subtitle.__get_srt_subs(path, housekeep=True)
+        return Subtitle._get_srt_subs(path, housekeep=True)
 
     @staticmethod
     def __convert_ass_to_subs(ass_file_path: str) -> SubRipFile:
@@ -573,14 +592,14 @@ class Subtitle(object):
             ass_file_path {string} -- The path to the Advanced SubStation Alpha v4.0+ subtitle file.
 
         Returns:
-            {list} -- A list of SubRipItems.
+            SubRipFile: A list of SubRipItems.
         """
 
         _, path = tempfile.mkstemp()
         path = "%s.srt" % path
         Utils.ass2srt(ass_file_path, path)
 
-        return Subtitle.__get_srt_subs(path, housekeep=True)
+        return Subtitle._get_srt_subs(path, housekeep=True)
 
     @staticmethod
     def __convert_microdvd_to_subs(microdvd_file_path: str) -> SubRipFile:
@@ -590,14 +609,14 @@ class Subtitle(object):
             microdvd_file_path {string} -- The path to the MicroDVD subtitle file.
 
         Returns:
-            {list} -- A list of SubRipItems.
+            SubRipFile: A list of SubRipItems.
         """
 
         _, path = tempfile.mkstemp()
         path = "%s.srt" % path
         Utils.microdvd2srt(microdvd_file_path, path)
 
-        return Subtitle.__get_srt_subs(path, housekeep=True)
+        return Subtitle._get_srt_subs(path, housekeep=True)
 
     @staticmethod
     def __convert_mpl2_to_subs(mpl2_file_path: str) -> SubRipFile:
@@ -607,14 +626,14 @@ class Subtitle(object):
             mpl2_file_path {string} -- The path to the MPL2 subtitle file.
 
         Returns:
-            {list} -- A list of SubRipItems.
+            SubRipFile: A list of SubRipItems.
         """
 
         _, path = tempfile.mkstemp()
         path = "%s.srt" % path
         Utils.mpl22srt(mpl2_file_path, path)
 
-        return Subtitle.__get_srt_subs(path, housekeep=True)
+        return Subtitle._get_srt_subs(path, housekeep=True)
 
     @staticmethod
     def __convert_tmp_to_subs(tmp_file_path: str) -> SubRipFile:
@@ -624,14 +643,14 @@ class Subtitle(object):
             tmp_file_path {string} -- The path to the TMP subtitle file.
 
         Returns:
-            {list} -- A list of SubRipItems.
+            SubRipFile: A list of SubRipItems.
         """
 
         _, path = tempfile.mkstemp()
         path = "%s.srt" % path
         Utils.tmp2srt(tmp_file_path, path)
 
-        return Subtitle.__get_srt_subs(path, housekeep=True)
+        return Subtitle._get_srt_subs(path, housekeep=True)
 
     @staticmethod
     def __convert_sami_to_subs(sami_file_path: str) -> SubRipFile:
@@ -641,13 +660,13 @@ class Subtitle(object):
             sami_file_path {string} -- The path to the SAMI subtitle file.
 
         Returns:
-            {list} -- A list of SubRipItems.
+            SubRipFile: A list of SubRipItems.
         """
 
         _, path = tempfile.mkstemp()
         Utils.sami2srt(sami_file_path, path)
 
-        return Subtitle.__get_srt_subs(path, housekeep=True)
+        return Subtitle._get_srt_subs(path, housekeep=True)
 
     @staticmethod
     def __convert_stl_to_subs(stl_file_path: str) -> SubRipFile:
@@ -657,13 +676,13 @@ class Subtitle(object):
             stl_file_path {string} -- The path to the STL subtitle file.
 
         Returns:
-            {list} -- A list of SubRipItems.
+            SubRipFile: A list of SubRipItems.
         """
 
         _, path = tempfile.mkstemp()
         Utils.stl2srt(stl_file_path, path)
 
-        return Subtitle.__get_srt_subs(path, housekeep=True)
+        return Subtitle._get_srt_subs(path, housekeep=True)
 
     @staticmethod
     def __convert_scc_to_subs(scc_file_path: str) -> SubRipFile:
@@ -673,13 +692,13 @@ class Subtitle(object):
             scc_file_path {string} -- The path to the SCC subtitle file.
 
         Returns:
-            {list} -- A list of SubRipItems.
+            SubRipFile: A list of SubRipItems.
         """
 
         _, path = tempfile.mkstemp()
         Utils.scc2srt(scc_file_path, path)
 
-        return Subtitle.__get_srt_subs(path, housekeep=True)
+        return Subtitle._get_srt_subs(path, housekeep=True)
 
     @staticmethod
     def __convert_sbv_to_subs(sbv_file_path: str) -> SubRipFile:
@@ -689,13 +708,13 @@ class Subtitle(object):
             sbv_file_path {string} -- The path to the SubViewer subtitle file.
 
         Returns:
-            {list} -- A list of SubRipItems.
+            SubRipFile: A list of SubRipItems.
         """
 
         _, path = tempfile.mkstemp()
         Utils.sbv2srt(sbv_file_path, path)
 
-        return Subtitle.__get_srt_subs(path, housekeep=True)
+        return Subtitle._get_srt_subs(path, housekeep=True)
 
     @staticmethod
     def __convert_ytt_to_subs(ytt_file_path: str) -> SubRipFile:
@@ -705,13 +724,28 @@ class Subtitle(object):
             ytt_file_path {string} -- The path to the YouTube transcript subtitle file.
 
         Returns:
-            {list} -- A list of SubRipItems.
+            SubRipFile: A list of SubRipItems.
         """
 
         _, path = tempfile.mkstemp()
         Utils.ytt2srt(ytt_file_path, path)
 
-        return Subtitle.__get_srt_subs(path, housekeep=True)
+        return Subtitle._get_srt_subs(path, housekeep=True)
+
+    @staticmethod
+    def __convert_json_raw_to_subs(json_file_path: str) -> SubRipFile:
+        """Convert a subtitle file from the JSON raw format to the SubRip format
+
+        Arguments:
+            json_file_path {string} -- The path to the JSON subtitle file.
+
+        Returns:
+            SubRipFile: A list of SubRipItems.
+        """
+        _, path = tempfile.mkstemp()
+        Utils.json2srt(json_file_path, path)
+
+        return Subtitle._get_srt_subs(path, housekeep=True)
 
     @staticmethod
     def __export_with_format(subs: List[SubRipItem], source_file_path: str, target_file_path: Optional[str], file_extension: str, suffix: str) -> None:
@@ -721,18 +755,6 @@ class Subtitle(object):
             )
         Subtitle.export_subtitle(source_file_path, subs, target_file_path)
 
-    @staticmethod
-    def __get_srt_subs(subrip_file_path: str, housekeep: bool = False) -> SubRipFile:
-        encoding = Utils.detect_encoding(subrip_file_path)
-        try:
-            subs = pysrt.open(subrip_file_path, encoding=encoding)
-        except Exception as e:
-            raise UnsupportedFormatException("Error occurred when loading subtitle from %s" % subrip_file_path) from e
-        finally:
-            if housekeep:
-                os.remove(subrip_file_path)
-        return subs
-
     @staticmethod
     def __save_subtitle_by_extension(file_extension: str,
                                      subs: List[SubRipItem],
@@ -846,7 +868,26 @@ class Subtitle(object):
                 Utils.srt2ytt(path, target_file_path)
             finally:
                 os.remove(path)
+        elif file_extension in Subtitle.JSON_RAW_EXTENSIONS:
+            try:
+                _, path = tempfile.mkstemp()
+                SubRipFile(subs).save(path, encoding=encoding)
+                Utils.srt2json(path, target_file_path)
+            finally:
+                os.remove(path)
         else:
             raise UnsupportedFormatException(
                 "Unknown subtitle format for file: {}".format(source_file_path)
             )
+
+    @staticmethod
+    def _get_srt_subs(subrip_file_path: str, housekeep: bool = False, encoding: Optional[str] = None) -> SubRipFile:
+        encoding = encoding or Utils.detect_encoding(subrip_file_path)
+        try:
+            subs = pysrt.open(subrip_file_path, encoding=encoding)
+        except Exception as e:
+            raise UnsupportedFormatException("Error occurred when loading subtitle from %s" % subrip_file_path) from e
+        finally:
+            if housekeep:
+                os.remove(subrip_file_path)
+        return subs
