@@ -8,7 +8,7 @@ import math
 import numpy as np
 import multiprocessing as mp
 
-from typing import List, Tuple, Optional
+from typing import List, Tuple, Optional, Union
 from .network import Network
 from .media_helper import MediaHelper
 from .hyperparameters import Hyperparameters
@@ -19,24 +19,27 @@ from .logger import Logger
 
 class Trainer(object):
     """Network trainer.
+
+    Arguments:
+        feature_embedder {Embedder.FeatureEmbedder} -- The feature embedder object.
+        feature_embedding_timeout {Union[int, float]} -- The maximum waiting time in seconds when embedding features of media files.
+        media_process_timeout {int} -- The maximum waiting time in seconds when processing media files.
+
+    Raises:
+        NotImplementedError: Thrown when any Trainer attributes are modified.
     """
 
-    EMBEDDING_TIMEOUT = 300  # time out for feature embedding of media files
-    __LOGGER = Logger().get_logger(__name__)
     __MAX_BYTES = 2 ** 31 - 1
 
-    def __init__(self, feature_embedder: FeatureEmbedder) -> None:
-        """Initialiser for the training process.
-
-        Arguments:
-            feature_embedder {Embedder.FeatureEmbedder} -- the feature embedder object.
-
-        Raises:
-            NotImplementedError -- Thrown when any Trainer attributes are modified.
-        """
-
+    def __init__(self,
+                 feature_embedder: FeatureEmbedder,
+                 feature_embedding_timeout: Union[int, float] = 300,
+                 media_process_timeout: int = 180) -> None:
         self.__feature_embedder = feature_embedder
+        self.__feature_embedding_timeout = feature_embedding_timeout
+        self.__media_helper = MediaHelper(media_process_timeout=media_process_timeout)
         self.__lock = threading.RLock()
+        self.__LOGGER = Logger().get_logger(__name__)
 
     def train(
         self,
@@ -62,7 +65,7 @@ class Trainer(object):
             weights_dir {string} -- The directory of the weights file.
             config_dir {string} -- The directory of the hyperparameter file where hyperparameters will be saved.
             logs_dir {string} -- The directory of the log file.
-            training_dump_dir {string} --  The directory of the training data dump file.
+            training_dump_dir {string}:  The directory of the training data dump file.
             hyperparameters {Hyperparameters} -- A configuration for hyperparameters used for training.
             training_log {string} -- The path to the log file of epoch results (default: {"training.log"}).
             resume {bool} -- True to continue with previous training result or False to start a new one (default: {False}).
@@ -78,7 +81,7 @@ class Trainer(object):
         if av_file_paths is None or subtitle_file_paths is None:
             # Load the data and labels dump from the disk
             training_dump = os.path.join(os.path.abspath(training_dump_dir), "training_dump.hdf5")
-            Trainer.__LOGGER.debug(
+            self.__LOGGER.debug(
                 "Resume training on data dump: ".format(
                     training_dump
                 )
@@ -101,7 +104,7 @@ class Trainer(object):
                     hyperparameters.to_file(hyperparams_filepath)
 
                     input_shape = (train_data_raw.shape[2], train_data_raw.shape[1])
-                    Trainer.__LOGGER.debug("input_shape: {}".format(input_shape))
+                    self.__LOGGER.debug("input_shape: {}".format(input_shape))
                     network = Network.get_network(input_shape, hyperparameters)
 
                 val_loss, val_acc = network.fit_with_generator(
@@ -134,7 +137,7 @@ class Trainer(object):
             train_data = train_data - np.mean(train_data, axis=0)
 
             input_shape = (train_data.shape[1], train_data.shape[2])
-            Trainer.__LOGGER.debug("input_shape: {}".format(input_shape))
+            self.__LOGGER.debug("input_shape: {}".format(input_shape))
 
             # Save hyperparameters before each new training
             hyperparameters.to_file(hyperparams_filepath)
@@ -150,9 +153,9 @@ class Trainer(object):
                 False,
             )
 
-        Trainer.__LOGGER.debug("val_loss: {}".format(min(val_loss)))
-        Trainer.__LOGGER.debug("val_acc: {}".format(max(val_acc)))
-        Trainer.__LOGGER.info(
+        self.__LOGGER.debug("val_loss: {}".format(min(val_loss)))
+        self.__LOGGER.debug("val_acc: {}".format(max(val_acc)))
+        self.__LOGGER.info(
             "Total training time: {}".format(
                 str(datetime.datetime.now() - training_start)
             )
@@ -178,10 +181,13 @@ class Trainer(object):
         Arguments:
             av_file_paths {list} -- A list of paths to the input audio/video files.
             subtitle_file_paths {list} -- A list of paths to the subtitle files.
-            training_dump_dir {string} --  The directory of the training data dump file.
+            training_dump_dir {string}:  The directory of the training data dump file.
             hyperparameters {Hyperparameters} -- A configuration for hyperparameters used for training.
             sound_effect_start_marker: {string} -- A string indicating the start of the ignored sound effect (default: {"("}).
             sound_effect_end_marker: {string} -- A string indicating the end of the ignored sound effect (default: {")"}).
+
+        Returns:
+            tuple: The valuation loss and accuracy.
         """
 
         training_dump = os.path.join(os.path.abspath(training_dump_dir), "training_dump.hdf5")
@@ -191,7 +197,7 @@ class Trainer(object):
                 labels_raw = hf["labels"]
 
                 input_shape = (train_data_raw.shape[2], train_data_raw.shape[1])
-                Trainer.__LOGGER.debug("input_shape: {}".format(input_shape))
+                self.__LOGGER.debug("input_shape: {}".format(input_shape))
 
                 val_loss, val_acc = Network.simple_fit_with_generator(
                     input_shape,
@@ -217,7 +223,7 @@ class Trainer(object):
             train_data = train_data - np.mean(train_data, axis=0)
 
             input_shape = (train_data.shape[1], train_data.shape[2])
-            Trainer.__LOGGER.debug("input_shape: {}".format(input_shape))
+            self.__LOGGER.debug("input_shape: {}".format(input_shape))
 
             val_loss, val_acc = Network.simple_fit(
                 input_shape,
@@ -233,6 +239,9 @@ class Trainer(object):
 
         Arguments:
             training_log {string} -- The path to the training log file.
+
+        Returns:
+            int: The number of finished epochs.
         """
         if not os.path.isfile(training_log):
             return 0
@@ -254,11 +263,14 @@ class Trainer(object):
         Arguments:
             av_file_paths {list} -- A list of paths to the input audio/video files.
             subtitle_file_paths {list} -- A list of paths to the subtitle files.
-            sound_effect_start_marker: {string} -- A string indicating the start of the ignored sound effect.
-            sound_effect_end_marker: {string} -- A string indicating the end of the ignored sound effect.
+            sound_effect_start_marker {string} -- A string indicating the start of the ignored sound effect.
+            sound_effect_end_marker {string} -- A string indicating the end of the ignored sound effect.
 
         Returns:
-            tuple -- The training data and labels.
+            tuple: The training data and labels.
+
+        Raises:
+            TerminalException: If the extraction is interrupted by user hitting the interrupt key.
         """
 
         train_data, labels = (
@@ -286,25 +298,25 @@ class Trainer(object):
                 for index in range(len(av_file_paths))
             ]
             try:
-                done, not_done = concurrent.futures.wait(futures, timeout=Trainer.EMBEDDING_TIMEOUT)
+                done, not_done = concurrent.futures.wait(futures, timeout=self.__feature_embedding_timeout)
             except KeyboardInterrupt:
                 for future in futures:
                     if not future.cancel():
-                        Trainer.__LOGGER.warning("Data and label extraction job cannot be cancelled")
+                        self.__LOGGER.warning("Data and label extraction job cannot be cancelled")
                 raise TerminalException("Data and label extraction interrupted by the user")
             for future in not_done:
                 # Log undone audio files and continue
                 try:
                     audio_file_path, subtitle_file_path = future.result()
-                    Trainer.__LOGGER.warning(
+                    self.__LOGGER.warning(
                         "Data and label extraction failed for: [Audio: {}, Subtitle: {}]".format(
                             audio_file_path, subtitle_file_path
                         )
                     )
                     if not future.cancel():
-                        Trainer.__LOGGER.warning("Data and label extraction job cannot be cancelled")
+                        self.__LOGGER.warning("Data and label extraction job cannot be cancelled")
                 except Exception as e:
-                    Trainer.__LOGGER.error(
+                    self.__LOGGER.error(
                         "Unexpected exception during data and label extraction: {} stacktrace: {}".format(
                             str(e), "".join(traceback.format_stack())
                         )
@@ -314,9 +326,9 @@ class Trainer(object):
         train_data = [x for x in train_data if x is not None]
         labels = [x for x in labels if x is not None]
 
-        train_data = np.concatenate(train_data)
-        labels = np.concatenate(labels)
-        Trainer.__LOGGER.debug(
+        train_data: np.ndarray = np.concatenate(train_data)  # type: ignore
+        labels: np.ndarray = np.concatenate(labels)  # type: ignore
+        self.__LOGGER.debug(
             "Data and labels extracted after {} seconds".format(
                 str(datetime.datetime.now() - extraction_start)
             )
@@ -329,20 +341,20 @@ class Trainer(object):
         index: int,
         av_file_path: str,
         subtitle_file_path: str,
-        train_data: np.ndarray,
-        labels: np.ndarray,
+        train_data: Union[np.ndarray, List],
+        labels: Union[np.ndarray, List],
         sound_effect_start_marker: Optional[str],
         sound_effect_end_marker: Optional[str]
     ) -> Tuple[str, str]:
         _, file_ext = os.path.splitext(av_file_path)
 
         try:
-            if file_ext not in MediaHelper.AUDIO_FILE_EXTENSION:
+            if file_ext not in self.__media_helper.AUDIO_FILE_EXTENSION:
                 t = datetime.datetime.now()
-                audio_file_path = MediaHelper.extract_audio(
+                audio_file_path = self.__media_helper.extract_audio(
                     av_file_path, True, 16000
                 )
-                Trainer.__LOGGER.debug(
+                self.__LOGGER.debug(
                     "- Audio extracted after {}".format(
                         str(datetime.datetime.now() - t)
                     )
@@ -362,12 +374,12 @@ class Trainer(object):
         # However, the training shall continue after expensive embedding on healthy media files.
         except Exception as e:
             # Log failed audio and subtitle files and continue
-            Trainer.__LOGGER.warning(
+            self.__LOGGER.warning(
                 "Exception: {}; stacktrace: {}".format(
                     str(e), "".join(traceback.format_stack())
                 )
             )
-            Trainer.__LOGGER.warning(
+            self.__LOGGER.warning(
                 "[Audio: {}, Subtitle: {}]".format(
                     audio_file_path, subtitle_file_path
                 )
