@@ -1,10 +1,12 @@
 import os
+import json
 import subprocess
 import pysubs2
 import requests
 import shutil
-import cchardet
+import chardet
 import shlex
+import pycountry
 
 from pycaption import (
     CaptionConverter,
@@ -23,6 +25,7 @@ from captionstransformer.core import Reader
 from captionstransformer.sbv import Reader as SbvReader, Writer as SbvWriter
 from captionstransformer.srt import Reader as SrtReader, Writer as SrtWriter
 from captionstransformer.transcript import Reader as TranscriptReader, Writer as TranscriptWriter
+from pysrt import SubRipFile, SubRipTime, SubRipItem
 from bs4 import BeautifulSoup
 from datetime import datetime
 from typing import Optional, TextIO, BinaryIO, Union, Callable, Any, Tuple, List, Dict
@@ -87,7 +90,7 @@ class Utils(object):
 
         _vtt_file_path = srt_file_path.replace(".srt", ".vtt") if vtt_file_path is None else vtt_file_path
         encoding = Utils.detect_encoding(srt_file_path)
-        command = "{0} -y -sub_charenc {1} -i '{2}' -f webvtt '{3}'".format(Utils.FFMPEG_BIN, encoding, srt_file_path, _vtt_file_path)
+        command = "{0} -y -sub_charenc {1} -i {2} -f webvtt {3}".format(Utils.FFMPEG_BIN, encoding, Utils.double_quoted(srt_file_path), Utils.double_quoted(_vtt_file_path))
         timeout_msg = "Timeout on converting SubRip to WebVTT: {}".format(srt_file_path)
         error_msg = "Cannot convert SubRip to WebVTT: {}".format(srt_file_path)
 
@@ -114,7 +117,7 @@ class Utils(object):
 
         _srt_file_path = vtt_file_path.replace(".vtt", ".srt") if srt_file_path is None else srt_file_path
         encoding = Utils.detect_encoding(vtt_file_path)
-        command = "{0} -y -sub_charenc {1} -i '{2}' -f srt '{3}'".format(Utils.FFMPEG_BIN, encoding, vtt_file_path, _srt_file_path)
+        command = "{0} -y -sub_charenc {1} -i {2} -f srt {3}".format(Utils.FFMPEG_BIN, encoding, Utils.double_quoted(vtt_file_path), Utils.double_quoted(_srt_file_path))
         timeout_msg = "Timeout on converting WebVTT to SubRip: {}".format(vtt_file_path)
         error_msg = "Cannot convert WebVTT to SubRip: {}".format(vtt_file_path)
 
@@ -480,6 +483,64 @@ class Utils(object):
 
         Utils.remove_trailing_newlines(srt_file_path, encoding)
 
+    @staticmethod
+    def srt2json(srt_file_path: str, json_raw_file_path: Optional[str] = None) -> None:
+        """Convert SubRip subtitles to JSON raw subtitles.
+
+        Arguments:
+            srt_file_path {string} -- The path to the SubRip file.
+            json_raw_file_path {string} -- The path to the JSON raw subtitle file.
+        """
+        Caption.text = property(lambda self: self._text, Utils._set_text_patch)
+        Reader.read = Utils._read_patch
+
+        file: Union[TextIO, BinaryIO]
+        encoding = Utils.detect_encoding(srt_file_path)
+
+        with open(srt_file_path, "r", encoding=encoding) as file:
+            srt_reader = SrtReader(file)
+            srt_reader.encoding = encoding
+            captions = srt_reader.read()
+            for caption in captions:
+                caption.encoding = encoding
+
+        if json_raw_file_path is None:
+            json_raw_file_path = srt_file_path.replace(".srt", ".json")
+
+        with open(json_raw_file_path, "w") as file:
+            timed_texts = []
+            for caption in captions:
+                timed_texts.append({
+                    "start": (caption.start - datetime(1900, 1, 1)).total_seconds(),
+                    "end": (caption.end - datetime(1900, 1, 1)).total_seconds(),
+                    "text": caption.text.rstrip("\r\n")
+                })
+            json.dump(timed_texts, file)
+
+    @staticmethod
+    def json2srt(json_raw_file_path: str, srt_file_path: Optional[str] = None) -> None:
+        """Convert JSON raw subtitles to SubRip subtitles.
+
+        Arguments:
+            json_raw_file_path {string} -- The path to the JSON raw subtitle file.
+            srt_file_path {string} -- The path to the SubRip file.
+        """
+
+        file: Union[TextIO, BinaryIO]
+        encoding = Utils.detect_encoding(json_raw_file_path)
+        subs = SubRipFile()
+        with open(json_raw_file_path, "r", encoding=encoding) as file:
+            for index, timed_text in enumerate(json.load(file), start=1):
+                start = SubRipTime(milliseconds=(timed_text["start"] * 1000))
+                end = SubRipTime(milliseconds=(timed_text["end"] * 1000))
+                subrip_item = SubRipItem(index, start=start, end=end, text=timed_text["text"])
+                subs.append(subrip_item)
+
+        if srt_file_path is None:
+            srt_file_path = json_raw_file_path.replace(".json", ".srt")
+
+        subs.save(srt_file_path, encoding=encoding)
+
     @staticmethod
     def extract_teletext_as_subtitle(ts_file_path: str, page_num: int, output_file_path: str, timeout_secs: int = 30) -> None:
         """Extract DVB Teletext from MPEG transport stream files and convert them into the output format.
@@ -491,7 +552,7 @@ class Utils(object):
             timeout_secs {int} -- The timeout in seconds on extraction {default: 30}.
         """
 
-        command = "{0} -y -fix_sub_duration -txt_page {1} -txt_format text -i '{2}' '{3}'".format(Utils.FFMPEG_BIN, page_num, ts_file_path, output_file_path)
+        command = "{0} -y -fix_sub_duration -txt_page {1} -txt_format text -i {2} {3}".format(Utils.FFMPEG_BIN, page_num, Utils.double_quoted(ts_file_path), Utils.double_quoted(output_file_path))
         timeout_msg = "Timeout on extracting Teletext from transport stream: {} on page: {}".format(ts_file_path, page_num)
         error_msg = "Cannot extract Teletext from transport stream: {} on page: {}".format(ts_file_path, page_num)
 
@@ -517,7 +578,7 @@ class Utils(object):
             timeout_secs {int} -- The timeout in seconds on extraction {default: 30}.
         """
 
-        command = "{0} -y -i '{1}' -map 0:s:{2} '{3}'".format(Utils.FFMPEG_BIN, mkv_file_path, stream_index, output_file_path)
+        command = "{0} -y -i {1} -map 0:s:{2} {3}".format(Utils.FFMPEG_BIN, Utils.double_quoted(mkv_file_path), stream_index, Utils.double_quoted(output_file_path))
         timeout_msg = "Timeout on extracting the subtitle from file: {} with stream index: {}".format(mkv_file_path, stream_index)
         error_msg = "Cannot extract the subtitle from file: {} with stream index: {}".format(mkv_file_path, stream_index)
 
@@ -566,10 +627,10 @@ class Utils(object):
             timeout_secs {int} -- The timeout in seconds on extraction {default: 30}.
 
         Returns:
-            bool -- True if the video contains embedded subtitles or False otherwise.
+            bool: True if the video contains embedded subtitles or False otherwise.
         """
 
-        command = "{0} -y -i '{1}' -c copy -map 0:s -f null - -v 0 -hide_banner".format(Utils.FFMPEG_BIN, video_file_path)
+        command = "{0} -y -i {1} -c copy -map 0:s -f null - -v 0 -hide_banner".format(Utils.FFMPEG_BIN, Utils.double_quoted(video_file_path))
         timeout_msg = "Timeout on detecting embedded subtitles from file: {}".format(video_file_path)
         error_msg = "Embedded subtitle detection failed for file: {}".format(video_file_path)
 
@@ -585,7 +646,7 @@ class Utils(object):
             subtitle_file_path {string} -- The path to the subtitle file.
 
         Returns:
-            string -- The string represent the encoding
+            str: The string represent the encoding
         """
 
         with open(subtitle_file_path, "rb") as file:
@@ -594,36 +655,39 @@ class Utils(object):
             # and hence this less memory-efficient solution:
             raw = b"".join(file.readlines())
 
-        detected = cchardet.detect(raw)
+        detected = chardet.detect(raw)
         detected = detected or {}
-        return detected["encoding"] if "encoding" in detected else None
+        return detected["encoding"] if "encoding" in detected and detected["encoding"] is not None else "utf-8"
 
     @staticmethod
-    def get_file_root_and_extension(file_path: str) -> Tuple[str, str]:
+    def get_file_root_and_extension(file_path: str) -> tuple:
         """Get the root path and the extension of the input file path.
 
+        Arguments:
+            file_path {str} -- the path to the file
+
         Returns:
-            tuple -- the root path and the extension of the input file path.
+            tuple: the root path and the extension of the input file path.
         """
 
         parts = os.path.abspath(file_path).split(os.extsep, 1)
         return parts[0], parts[1]
 
     @staticmethod
-    def get_stretch_language_codes() -> List[str]:
+    def get_stretch_language_codes() -> list:
         """Get language codes used by stretch.
 
         Returns:
-            list -- A list of language codes derived from ISO 639-3.
+            list: A list of language codes derived from ISO 639-3.
         """
         return Language.ALLOWED_VALUES
 
     @staticmethod
-    def get_misc_language_codes() -> List[str]:
+    def get_misc_language_codes() -> list:
         """Get all known language codes.
 
         Returns:
-            list -- A list of all known language codes.
+            list: A list of all known language codes.
         """
         return Language.ALLOWED_VALUES + \
             ['CELTIC', 'NORTH_EU', 'NORWAY', 'ROMANCE', 'SAMI', 'SCANDINAVIA', 'aav', 'aed', 'afa', 'alv', 'art', 'ase',
@@ -636,11 +700,11 @@ class Utils(object):
              'tvl', 'tzo', 'umb', 'urj', 'vsl', 'wal', 'war', 'wls', 'yap', 'yua', 'zai', 'zle', 'zls', 'zlw', 'zne']
 
     @staticmethod
-    def get_language_table() -> List[str]:
+    def get_language_table() -> list:
         """Get all known language codes and their human-readable versions.
 
         Returns:
-            list -- A list of all known language codes and their human-readable versions.
+            list: A list of all known language codes and their human-readable versions.
         """
         return list(map(lambda line: line.replace("\t", "  "), Language.CODE_TO_HUMAN_LIST)) + \
             ['CELTIC', 'NORTH_EU', 'NORWAY', 'ROMANCE', 'SAMI', 'SCANDINAVIA', 'aav', 'aed', 'afa', 'alv', 'art', 'ase',
@@ -652,6 +716,42 @@ class Utils(object):
              'sem', 'sit', 'sla', 'srn', 'ssp', 'swc', 'taw', 'tdt', 'tiv', 'tll', 'toi', 'tpi', 'trk', 'tum', 'tut',
              'tvl', 'tzo', 'umb', 'urj', 'vsl', 'wal', 'war', 'wls', 'yap', 'yua', 'zai', 'zle', 'zls', 'zlw', 'zne']
 
+    @staticmethod
+    def get_iso_639_alpha_2(language_code: str) -> str:
+        """Find the alpha 2 language code based on an alpha 3 one.
+
+        Arguments:
+            language_code {string} -- An alpha 3 language code derived from ISO 639-3.
+
+        Returns:
+            str: The alpha 2 language code if exists otherwise the alpha 3 one.
+        """
+
+        lang = pycountry.languages.get(alpha_3=language_code)
+        if lang is None:
+            return language_code
+        elif hasattr(lang, "alpha_2"):
+            return lang.alpha_2
+        else:
+            return lang.alpha_3
+
+    @staticmethod
+    def format_timestamp(seconds: float) -> str:
+        assert seconds >= 0, "non-negative timestamp expected"
+        milliseconds = round(seconds * 1000.0)
+        hours = milliseconds // 3_600_000
+        milliseconds -= hours * 3_600_000
+        minutes = milliseconds // 60_000
+        milliseconds -= minutes * 60_000
+        seconds = milliseconds // 1_000
+        milliseconds -= seconds * 1_000
+        hours_marker = f"{hours:02d}:"
+        return f"{hours_marker}{minutes:02d}:{seconds:02d},{milliseconds:03d}"
+
+    @staticmethod
+    def double_quoted(s: str) -> str:
+        return "\"{}\"".format(s.replace('"', "\\\""))
+
     @staticmethod
     def __convert_subtitle(source_file_path: str, source_ext: str, target_file_path: Optional[str], target_ext: str, format: str, frame_rate: Optional[float] = None) -> Tuple[str, str]:
         encoding = Utils.detect_encoding(source_file_path)
